const DCA_NOTEBOOK_ID = "ipython-main-app";
const DCA = { frontend: 'classic', assistants: {}, toArray: (x) => x, toObject: (x) => x };

window.DCA = DCA;


// these wrapper classes mimic the jupyter lab interface for cells

class CellMetadataWrapper {
    constructor(cell) {
        this.cell = cell;
    }
    set(key, value) {
        this.cell.metadata[key] = value;
    }
    get(key) {
        return this.cell.metadata[key];
    }
}

class CellValueWrapper {
    constructor(cell) {
        this.cell = cell;
    }

    get text() {
        return this.cell.get_text();
    }

    set text(value) {
        this.cell.set_text(value);
    }
}

class CellModelWrapper {
    constructor(cell) {
        this.cell = cell;
    }
    get id() {
        return this.cell.id;
    }
    set id(value) {
        this.cell.id = value;
    }
    get type() {
        return this.cell.cell_type;
    }
    get value() {
        return new CellValueWrapper(this.cell);
    }
    get metadata() {
        return new CellMetadataWrapper(this.cell);
    }
}

class CellWrapper {
    constructor(cell) {
        this.cell = cell;
    }
    get model() {
        return new CellModelWrapper(this.cell);
    }
    get node() {
        return this.cell.input ? this.cell.input[0] : this.cell.element[0];
    }
}

define(['jquery', 'base/js/namespace', 'base/js/events', 'nbextensions/domino-code-assist/mixpanel'], function ($, Jupyter, events, initMixpanel) {
    "use strict";

    const kernelReady = Jupyter.notebook.kernel.info_reply && Jupyter.notebook.kernel.info_reply.status === "ok"
        ? Promise.resolve()
        : new Promise((resolve, reject) => {
            events.on("kernel_ready.Kernel", () => {
                resolve();
            });
        });

    DCA.getNotebookId = () => {
        return DCA_NOTEBOOK_ID;
    }
    DCA.getCells = () => {
        return Jupyter.notebook.get_cells().map(cell => new CellWrapper(cell));
    }
    DCA.changeCellType = (notebookId, cell, cellType) => {
        if (cellType === "code") {
            Jupyter.notebook.to_code();
        } else if (cellType === "markdown") {
            Jupyter.notebook.to_markdown();
        } else {
            throw new Error("Unknown cell type: " + cellType);
        }
    }
    DCA.insertCellAbove = (notebookId, cell, value) => {
        const i = Jupyter.notebook.find_cell_index(cell.cell);
        Jupyter.notebook.select(i);
        Jupyter.notebook.insert_cell_above(value.type_ || "code", i);
        const newCell = Jupyter.notebook.get_cell(i);
        newCell.set_text(value.code);
        if (value.meta) {
            newCell.metadata["assistant"] = value.meta;
        }
        Jupyter.notebook.select(i);
        Jupyter.notebook.execute_selected_cells();
        Jupyter.notebook.select(i+1);
    }
    DCA.insertCellsBelow = (notebookId, cell, cells) =>{
        let i = Jupyter.notebook.find_cell_index(cell.cell);
        Jupyter.notebook.select(i);
        cells.forEach(({code, meta, type_, id}) => {
            Jupyter.notebook.insert_cell_below(type_ || "code", i);
            i += 1;
            const newCell = Jupyter.notebook.get_cell(i);
            newCell.set_text(code);
            if (meta) {
                newCell.metadata["assistant"] = meta;
            }
            if (id) {
                newCell.id = id;
            }
            Jupyter.notebook.select(i);
            Jupyter.notebook.execute_selected_cells();
        })
        Jupyter.notebook.get_cell(i).focus_cell();
    }
    DCA.runAndAdvance = (notebookId) => {
        Jupyter.notebook.execute_cell_and_select_below();
    }
    DCA.getNotebookPath = (notebookId) => {
        return Jupyter.notebook.notebook_path;
    }
    DCA.saveNotebook = async (notebookId) => {
        try {
            await Jupyter.notebook.save_notebook();
            return true;
        } catch (e) {
            return false;
        }
    }
    DCA.activate = (notebookId, cell) => {
        cell.cell.focus_cell();
    }
    DCA.kernelBusy = (notebookId) => {
        return Jupyter.notebook.kernel_busy;
    }
    DCA.getFormatVersion = (notebookId) => {
        return [Jupyter.notebook.nbformat, Jupyter.notebook.nbformat_minor];
    }
    var init_assistant = function () {
        const cell = Jupyter.notebook.insert_cell_above('code');
        cell.set_text('# this code is generated by the Domino Code Assist toolbar button\nimport domino_code_assist as dca\ndca.init()')
        cell.execute()
        Jupyter.notebook.metadata["dca-init"] = true;
    }
    const toggle_assistant = function () {
        const assistant_cell = find_assistant();
        if (assistant_cell) {
            Jupyter.notebook.delete_cell(Jupyter.notebook.find_cell_index(assistant_cell));
        } else {
            init_assistant();
        }
    }
    const find_assistant = function () {
        return Jupyter.notebook.get_cells().find(cell => cell.get_text().includes('# this code is generated by the Domino Code Assist toolbar button'));
    }
    var load_ipython_extension = function () {
        const btn_group = Jupyter.toolbar.add_buttons_group([
        {
            id: 'init_assistant',
            label: 'Domino Code Assist',
            icon: 'fa-gear',
            callback: toggle_assistant
        }]);
        btn_group.find("button").css("background-color", "rgb(45, 113, 199);")

        btn_group.find("button").prop("disabled", true);
        kernelReady.then(() => {
            btn_group.find("button").prop("disabled", false)
        });

        // maybe a better background?
        // btn_group.find("button").css("background-color", "#1E2B42;")
        btn_group.find("button").css("color", "white")
        const svgData = btoa('<svg width="600" height="600" viewBox="0 0 600 600" fill="none" xmlns="http://www.w3.org/2000/svg">\n' +
        '<path d="M280.19 142.036C282.913 150.121 288.605 156.721 296.196 160.516C300.733 162.743 305.6 163.898 310.468 163.898C313.851 163.898 317.315 163.321 320.616 162.248L470.852 112.17C487.599 106.642 496.592 88.4924 491.064 71.7447C485.536 54.997 467.303 46.0044 450.636 51.532L300.403 101.61C283.738 107.137 274.663 125.288 280.19 142.036Z" fill="white"/>\n' +
        '<path d="M318.056 481.439C321.851 473.769 322.511 465.186 319.788 457.017C314.261 440.354 296.111 431.278 279.363 436.806L129.129 486.883C121.043 489.608 114.443 495.299 110.648 502.89C106.853 510.481 106.193 519.144 108.916 527.308C113.371 540.674 125.828 549.173 139.194 549.173C142.576 549.173 145.959 548.68 149.259 547.525L299.493 497.446C307.578 494.721 314.178 489.03 317.973 481.439H318.056Z" fill="white"/>\n' +
        '<path d="M174.588 202.183C179.125 204.493 183.992 205.565 188.778 205.565C200.493 205.565 211.795 199.13 217.405 187.91L288.191 46.2558C291.986 38.6657 292.646 30.0031 289.924 21.8355C287.201 13.7504 281.509 7.15028 273.918 3.35523C258.161 -4.48236 238.938 1.87022 231.101 17.6279L160.315 159.282C152.477 175.04 158.83 194.263 174.588 202.1V202.183Z" fill="white"/>\n' +
        '<path d="M425.308 396.871C417.718 393.076 408.973 392.416 400.888 395.139C392.803 397.861 386.203 403.554 382.408 411.144L311.622 552.797C307.826 560.388 307.166 569.05 309.889 577.219C312.612 585.388 318.304 591.903 325.894 595.699C330.432 597.925 335.217 599.08 340.167 599.08C343.549 599.08 347.015 598.502 350.314 597.431C358.4 594.707 365 589.016 368.795 581.424L439.581 439.772C443.375 432.182 444.036 423.519 441.313 415.351C438.59 407.266 432.898 400.666 425.308 396.871Z" fill="white"/>\n' +
        '<path d="M102.067 299.128C106.522 312.493 118.98 320.991 132.428 320.991C135.728 320.991 139.193 320.496 142.493 319.341C159.158 313.813 168.233 295.663 162.706 278.915L112.628 128.681C107.1 112.016 88.9499 102.858 72.2022 108.468C55.5369 113.996 46.4619 132.146 51.9896 148.894L102.067 299.128Z" fill="white"/>\n' +
        '<path d="M497.827 299.945C492.299 283.197 474.145 274.205 457.398 279.732C449.313 282.455 442.714 288.147 438.919 295.738C435.124 303.328 434.464 311.99 437.186 320.158L487.264 470.392C491.721 483.758 504.179 492.257 517.541 492.257C520.926 492.257 524.308 491.759 527.609 490.604C544.273 485.076 553.35 466.927 547.822 450.179L497.743 299.945H497.827Z" fill="white"/>\n' +
        '<path d="M174.01 442.593C177.392 442.593 180.857 442.015 184.157 440.94C192.242 438.219 198.843 432.527 202.637 424.936C206.432 417.347 207.092 408.684 204.37 400.516C201.648 392.431 195.955 385.831 188.365 382.036L46.7104 311.25C30.9528 303.413 11.7301 309.765 3.8925 325.523C0.0974548 333.195 -0.562554 341.775 2.15998 349.943C4.88251 358.028 10.5751 364.628 18.1652 368.423L159.82 439.209C164.357 441.438 169.225 442.593 174.092 442.593H174.01Z" fill="white"/>\n' +
        '<path d="M597.816 249.22C595.096 241.136 589.405 234.535 581.814 230.74L440.159 159.954C424.401 152.117 405.178 158.47 397.341 174.227C393.546 181.9 392.886 190.48 395.608 198.648C398.331 206.732 404.023 213.333 411.614 217.128L553.266 287.914C557.806 290.224 562.673 291.296 567.456 291.296C579.174 291.296 590.477 284.861 596.084 273.641C599.88 266.051 600.541 257.388 597.816 249.22Z" fill="white"/>\n' +
        '</svg>')
        btn_group.find("i").replaceWith(`<img style="height: 1em; margin-top: -1px;" src="data:image/svg+xml;base64,${svgData}"/>`)
        // debugging option
        // window.btn_group = btn_group;

        initMixpanel()

    };
    return {
        load_ipython_extension: load_ipython_extension
    };
});
